from __future__ import unicode_literals


class MongoCounterError(Exception):
    pass


class MongoCounter(object):
    def __init__(self, collection, counter_name):
        self.collection = collection
        self.counter_name = counter_name

    def get_value(self):
        doc = self.collection.find_one(
            {'_id': self.counter_name}, {'value': 1})
        if not doc:
            raise MongoCounterError(
                'No such counter "{}" in {}'.format(
                    self.counter_name, self.collection.full_name))
        return doc['value']

    def next_value(self):
        doc = self.collection.find_and_modify(
            {'_id': self.counter_name},
            {'$inc': {'value': 1}},
            new=True,
            upsert=True,
        )
        return doc['value']



from ognom.connection import ConnectionManager


def pytest_configure(config):
    connection_manager = ConnectionManager()
    connection_manager.connect({'main': {
        'name': 'master_common_test',
        'args': ['127.0.0.1:27017']}})




import unittest
from decimal import Decimal

import pytest

from ognom.fields import (
    DocumentField, URLField, ValidationError, HTTPField, DecimalField)
from tests.common import BaseDoc


class TestDocumentField(unittest.TestCase):
    def test_should_accept_document_class_as_model_class(self):
        field = DocumentField(BaseDoc)
        assert field.model_class == BaseDoc

    def test_should_accept_string_as_model_class(self):
        field = DocumentField('tests.common.BaseDoc')
        assert field.model_class == BaseDoc


class TestURLField(unittest.TestCase):

    def setUp(self):
        self.field = URLField()

    def test_validate_wrong_urls(self):
        wrong_urls = [
            'aaaaaaaa',
            'some.url.domain',
            '://somestring'
        ]
        for url in wrong_urls:
            self.assertRaises(ValidationError, self.field.validate, url)

    def test_validate_regular_url(self):
        valid_urls = [
            'http://some.domain.ru',
            'ftp://some.domain.ru',
            'http://some.domain.ru:8080',
            'http://some.domain.ru:8011/path',
            'http://some.domain.ru:9001/path&param1=value1',
        ]
        for url in valid_urls:
            self.assertIsNone(self.field.validate(url))


class TestHTTPField(unittest.TestCase):

    def setUp(self):
        self.field = HTTPField()

    def test_validate_wrong_urls(self):
        wrong_urls = [
            'aaaaaaaa',
            'some.url.domain',
            '://somestring',
            'ftp://some.domain.ru',
            'ftps://some.domain.ru'
        ]
        for url in wrong_urls:
            self.assertRaises(ValidationError, self.field.validate, url)

    def test_validate_regular_url(self):
        valid_urls = [
            'http://some.domain.ru',
            'https://some.domain.ru:8080',
            'http://some.domain.ru:8011/path',
            'https://some.domain.ru:9001/path&param1=value1',
        ]
        for url in valid_urls:
            self.assertIsNone(self.field.validate(url))


class TestDecimalField(unittest.TestCase):
    def setUp(self):
        self.field = DecimalField()

    def test_should_convert_to_string_when_storing(self):
        value = self.field.to_mongo(Decimal('3.14'))
        assert value == '3.14'
        value = self.field.to_mongo(Decimal('3.1415926535897932384'))
        assert value == '3.1415926535897932384'

    def test_should_restore_to_decimal(self):
        value = self.field.from_mongo('3.14')
        assert value == Decimal('3.14')
        value = self.field.from_mongo('3.1415926535897932384')
        assert value == Decimal('3.1415926535897932384')

    def test_validate_non_compatible_decimals(self):
        wrong_values = [
            'string',
            '1/2',
        ]
        for value in wrong_values:
            self.assertRaises(ValidationError, self.field.validate, value)

    def test_validate_regular_url(self):
        valid_urls = [
            1,
            50.51,
            Decimal('1'),
            Decimal('50.51'),
        ]
        for url in valid_urls:
            self.assertIsNone(self.field.validate(url))



# coding: utf8
from __future__ import unicode_literals
from collections import namedtuple

from bson import ObjectId
from bson.errors import InvalidId
from pymongo.cursor import Cursor
from pymongo.errors import ConnectionFailure
from six import string_types

from ognom.fields import DateTimeField
from ognom.connection import ConnectionManager
from ognom.document import Document

_IndexSpec = namedtuple(
    '_IndexSpec', [
        'name', 'spec', 'background', 'unique', 'expire_after_seconds'])


class CursorWrapper(object):
    def __init__(self, cursor, serialize=None):
        self.cursor = cursor
        self.serialize = serialize

    def __getitem__(self, item):
        item_or_slice = self.cursor[item]
        if not self.serialize:
            return item_or_slice

        if isinstance(item_or_slice, Cursor):
            return [self.serialize(i) for i in item_or_slice]

        return self.serialize(item_or_slice)

    def __getattr__(self, item):
        return getattr(self.cursor, item)

    def __iter__(self):
        return (
            self.serialize(r) if self.serialize else r
            for r in self.cursor
        )

    def skip(self, count):
        self.cursor.skip(count)
        return self

    def limit(self, count):
        self.cursor.limit(count)
        return self

    def next(self):
        next_val = next(self.cursor)
        if self.serialize:
            next_val = self.serialize(next_val)
        return next_val

    def as_list(self):
        return list(self)


class Collection(object):
    def __init__(self, db_name, collection_name=None, indexes=None):
        """
        Collection class. Stores/retrieves objects from database.
        :param db_name: name of the database;
        :param collection_name: collection name, if empty will be
            populated with pluaralyzed class name;
        :param indexes: list of indexes.
        """
        self.db_name = db_name
        self.collection_name = collection_name
        if indexes is None:
            indexes = []
        self.indexes = indexes

        # will be populated later
        self.model_class = None
        self._collection = None

    def serialize(self, result):
        if isinstance(result, (tuple, list)):
            return [self.model_class.from_mongo(v) for v in result]
        return self.model_class.from_mongo(result)

    @property
    def collection(self):
        if not self.collection_name:
            self.collection_name = '{}'.format(
                self.model_class.__name__.lower())

        if not self._collection:
            db = ConnectionManager.get_database(self.db_name)
            if not db:
                raise ConnectionFailure(
                    'No connection for {}'.format(self.db_name))
            self._collection = getattr(db, self.collection_name)
        return self._collection

    # CRUD
    def find(self, spec=None, fields=None, skip=None, limit=None, sort=None,
             as_dict=False, **kwargs):
        find_specs = {
            name: val
            for name, val
            in (('spec', spec), ('fields', fields), ('skip', skip),
                ('limit', limit), ('sort', sort))
            if val
        }
        find_specs.update(kwargs)
        result = self.collection.find(**find_specs)
        if not as_dict:
            result = CursorWrapper(result, self.serialize)
        return result

    def get(self, spec_or_id=None, fields=None):
        if spec_or_id and not isinstance(spec_or_id, dict):
            spec_or_id = ObjectId(spec_or_id)
        get_specs = {
            name: val for name, val
            in (('spec_or_id', spec_or_id), ('fields', fields))
            if val
        }
        return self.serialize(self.collection.find_one(**get_specs))

    def get_or_raise(self, spec_or_id=None, fields=None):
        try:
            result = self.get(spec_or_id, fields)
        # TypeError is required because of bug in bson lib.
        # It throws TypeError when unicode is passed to ObjectId constructor.
        except (InvalidId, TypeError):
            raise self.model_class.DoesNotExist()

        if not result:
            raise self.model_class.DoesNotExist()

        return result

    def count(self, spec=None):
        qs = self.collection.find(spec)
        count = qs.count()
        return count

    def create(self, payload):
        instance = self.model_class(**payload)
        self.save(instance)
        return instance

    def update(self, spec_or_id, document, multi=False, upsert=False, w=1):
        """
        Important! document here can be either instance of updated document,
        or a "document" in mongo update notation,
        for example {'$set': {'field1': 100}}.
        @param spec_or_id:
        @param document:
        @return:
        """
        is_doc = False
        if not isinstance(spec_or_id, dict):
            spec_or_id = {'_id': ObjectId(spec_or_id)}
        if isinstance(document, Document):
            document_as_dict = document.to_mongo()
            is_doc = True
        else:
            document_as_dict = document
        result = self.collection.update(
            spec_or_id, document_as_dict, upsert=upsert, multi=multi, w=w)
        if is_doc and 'upserted' in result:
            document._id = result['upserted']
        return result

    def save(self, doc, w=1):
        doc.validate()
        result = self.collection.save(doc.to_mongo(), w=w)
        if not doc._id:
            doc._id = result
        return result

    def insert(self, doc_or_docs, **kwargs):
        if isinstance(doc_or_docs, Document):
            doc_or_docs = [doc_or_docs]
        else:
            doc_or_docs = list(doc_or_docs)

        for doc in doc_or_docs:
            doc.validate()
        entities = [(doc, doc.to_mongo()) for doc in doc_or_docs]
        result = self.collection.insert(
            [doc_as_dict for _, doc_as_dict in entities], **kwargs
        )
        if kwargs.get('manipulate') is not False:
            for doc, doc_as_dict in entities:
                doc._id = doc_as_dict['_id']
        return result

    def find_and_modify(self, spec_or_id, document, **kwargs):
        """
        @param spec_or_id: filter for updating (default is ``{}``)
        @param document: either instance of updated document, or a dict in
                mongo update notation, for example {'$set': {'field1': 100}}
        @param kwargs: all kwargs, which supported by
        ``pymongo.Collection.find_and_modify`` method:
            - `upsert`: insert if object doesn't exist (default ``False``)
            - `sort`: a list of (key, direction) pairs specifying the sort
              order for this query. See :meth:`~pymongo.cursor.Cursor.sort`
              for details.
            - `full_response`: return the entire response object from the
              server (default ``False``)
            - `remove`: remove rather than updating (default ``False``)
            - `new`: return updated rather than original object
              (default ``False``)
            - `fields`: see second argument to :meth:`find` (default all)
            - and also all generic pymongo kwargs as w, j and etc...
        @return: instance of some ``Document`` descendant
        """
        if not isinstance(spec_or_id, dict):
            spec_or_id = {'_id': ObjectId(spec_or_id)}
        if isinstance(document, Document):
            document_as_dict = document.to_mongo()
        else:
            document_as_dict = document
        result = self.collection.find_and_modify(
            spec_or_id, document_as_dict, **kwargs)
        if kwargs.get('full_response'):
            return result
        return self.serialize(result)

    def remove(self, spec_or_id=None, w=1):
        if isinstance(spec_or_id, Document):
            spec_or_id = spec_or_id._id
        if (spec_or_id and not isinstance(spec_or_id, dict) and
                isinstance(spec_or_id, string_types)):
            spec_or_id = ObjectId(spec_or_id)
        return self.collection.remove(spec_or_id, w=w)

    def aggregate(self, pipeline, **kwargs):
        default_kwargs = {'cursor': {}}
        default_kwargs.update(kwargs)
        return self.collection.aggregate(pipeline, **default_kwargs)

    def generate_next_id(self, field):
        result = self.aggregate([{
            '$group':  {
                '_id': '0',
                'max': {'$max': u'${}'.format(field)}
            }
        }])

        result = result[0]['max'] if result else 0
        max_id = int(result) if result else 0
        return max_id + 1

    def synchronize_indexes(self):
        def generate_names(indexes):
            for index in indexes:
                index['name'] = '_'.join(
                    str(i) for j in index['index'] for i in j)
            return indexes

        def new_index_to_set(indexes):
            return set(
                _IndexSpec(
                    index['name'],
                    tuple(index['index']),
                    index.get('background', True),
                    index.get('unique', False),
                    index.get('expire_after_seconds')
                )
                for index in indexes
            )

        def old_index_to_set(indexes):
            return set(
                _IndexSpec(
                    index,
                    tuple(indexes[index]['key']),
                    indexes[index].get('background', True),
                    indexes[index].get('unique', False),
                    indexes[index].get('expireAfterSeconds')
                )
                for index in indexes.keys()
            )

        result = {'indexes_to_ensure': [], 'indexes_to_drop': []}

        if self.indexes:
            new_indexes = new_index_to_set(generate_names(self.indexes))

            old_indexes = self.collection.index_information()
            old_indexes.pop('_id_', None)
            old_indexes = old_index_to_set(old_indexes)

            indexes_to_ensure = new_indexes - old_indexes
            indexes_to_drop = old_indexes - new_indexes

            for index in indexes_to_drop:
                self.collection.drop_index(index[0])
                result['indexes_to_drop'].append(index[0])

            for index in indexes_to_ensure:
                if index.expire_after_seconds:
                    field_name = index.spec[0][0]
                    if (len(index.spec) > 1 or
                            not isinstance(getattr(getattr(
                                self, 'model_class'), field_name),
                                DateTimeField) or
                            not isinstance(index.expire_after_seconds, int)):
                        raise TypeError(
                            'Incorrect expire_after_seconds '
                            'assignment to collection')
                index_opts = {
                    'name': index.name,
                }
                if index.background:
                    index_opts['background'] = index.background
                if index.unique:
                    index_opts['unique'] = index.unique
                if index.expire_after_seconds:
                    index_opts['expireAfterSeconds'] = \
                        index.expire_after_seconds
                res_ind = self.collection.ensure_index(
                    list(index.spec), **index_opts)
                result['indexes_to_ensure'].append(res_ind)
        return result



# coding: utf8
import uuid
import unittest
import inspect
from datetime import datetime, timedelta
from collections import namedtuple

import pytest
from bson.objectid import ObjectId
from pymongo.errors import DuplicateKeyError
from six import string_types
from ognom.collection import Collection
from ognom.document import ObjectDoesNotExist

from ognom.validators import EmailValidator
from ognom.fields import (
    StringField, ObjectIdField, DateTimeField, ValidationError, UUIDField,
    ListField, DictField, DocumentField, BooleanField, IntField, GenericField)
from ognom._registry import documents_registry
from tests.common import BaseDoc

_ContactStatus = namedtuple(
    'ContactStatus', ['Awaiting', 'Done', 'InProcess', 'Failed'])
ContactStatus = _ContactStatus('awaiting', 'done', 'in-process', 'failed')


class TestDocument(unittest.TestCase):

    def test_attribute_getting(self):
        class TD(BaseDoc):
            field1 = StringField()
        td = TD(field1='test_string')
        assert td.field1 == 'test_string'

    def test_attribute_setting(self):
        class TD(BaseDoc):
            field1 = StringField()
        td = TD()
        td.field1 = 'test_string'
        assert td.field1 == 'test_string'

    def test_validating(self):
        class TD(BaseDoc):
            field1 = ObjectIdField()
        td = TD(field1='test_string')
        self.assertRaises(ValidationError, td.validate)

    def test_none_should_be_treated_as_value_absence(self):
        class TD(BaseDoc):
            field1 = StringField(required=True)
        td = TD(field1=None)
        with pytest.raises(ValidationError):
            td.validate()

    def test_none_should_pass_optional_validators(self):
        class TD(BaseDoc):
            field1 = StringField(validators=[EmailValidator()])
        td = TD(field1=None)
        td.validate()
        assert td.field1 is None

    def test_required_field(self):
        class TD(BaseDoc):
            field1 = StringField(required=True)
        td = TD()
        self.assertRaises(ValidationError, td.validate)

    def test_defaults(self):
        class TD(BaseDoc):
            field1 = StringField(required=True, default='test_string')
            field2 = BooleanField(required=True, default=False)
            field3 = StringField(default='test_string')

        td = TD()
        assert td.field1 == 'test_string'
        assert td.field2 is False
        assert td.field3 == 'test_string'

    def test_to_mongo(self):
        class TD(BaseDoc):
            field1 = StringField(required=True, default='test_string')
            field2 = DateTimeField(default=datetime.now)
            field3 = UUIDField(default=uuid.uuid4)
        td = TD()
        result = td.to_mongo()
        assert result['field1'] == 'test_string'
        assert isinstance(result['field2'], datetime)
        assert isinstance(result['field3'], uuid.UUID)

    def test_jsonify(self):
        class TD(BaseDoc):
            field1 = StringField(required=True, default='test_string')
            field2 = DateTimeField(default=datetime.now)
            field3 = UUIDField(default=uuid.uuid4)
            field4 = ObjectIdField(default=ObjectId)
        td = TD()
        result = td.jsonify()
        assert result['field1'] == 'test_string'
        assert isinstance(result['field2'], string_types)
        assert isinstance(result['field3'], string_types)
        assert isinstance(result['field4'], string_types)

    def test_list_field_to_mongo(self):
        class TD(BaseDoc):
            field1 = ListField(IntField())
        td = TD(field1=[1, 2, 3])
        result = td.to_mongo()
        assert result['field1'], [1, 2 == 3]

    def test_list_field_from_mongo(self):
        class TD(BaseDoc):
            field1 = ListField(StringField())
        result = TD.from_mongo({'field1': ['a', 'b', 3]})
        assert result.field1, ['a', 'b', '3']

    def test_dict_field_to_mongo(self):
        class TD(BaseDoc):
            field1 = DictField(IntField())
        td = TD(field1={'a': 1, 'b': 1})
        result = td.to_mongo()
        assert result['field1'], {'a': 1, 'b': 1}

    def test_document_field_to_mongo(self):
        class ITD(BaseDoc):
            field1 = StringField(default='test_string')

        class TD(BaseDoc):
            field1 = DocumentField(ITD)
        td = TD(field1=ITD())
        result = td.to_mongo()
        assert result['field1'] == {'field1': 'test_string'}

    def test_list_of_document_field_to_mongo(self):
        class ITD(BaseDoc):
            field1 = StringField(default='ts')

        class TD(BaseDoc):
            field1 = ListField(DocumentField(ITD))
        td = TD(field1=[ITD(), ITD(), ITD()])
        result = td.to_mongo()
        self.assertEqual(
            result['field1'],
            [{'field1': 'ts'}, {'field1': 'ts'}, {'field1': 'ts'}])

    def test_dict_of_document_field_to_mongo(self):
        class ITD(BaseDoc):
            field1 = StringField(default='ts')

        class TD(BaseDoc):
            field1 = DictField(DocumentField(ITD))
        td = TD(field1={'a': ITD(), 'b': ITD()})
        result = td.to_mongo()
        self.assertEqual(
            result['field1'],
            {'a': {'field1': 'ts'}, 'b': {'field1': 'ts'}})

    def test_dict_from_mongo(self):
        class TD(BaseDoc):
            field1 = DictField(GenericField())
        result = TD.from_mongo({'field1': {'a': 'b'}})
        assert result.field1['a'] == 'b'

    def test_dict_of_document_field_jsonify(self):
        dt_now = datetime.now()
        dt_now_str = dt_now.strftime('%Y-%m-%dT%H:%M:%S')

        class ITD(BaseDoc):
            field1 = DateTimeField(default=dt_now)

        class TD(BaseDoc):
            field1 = DictField(DocumentField(ITD))

        td = TD(field1={'a': ITD(), 'b': ITD()})
        result = td.jsonify()
        self.assertEqual(
            result['field1'],
            {'a': {'field1': dt_now_str}, 'b': {'field1': dt_now_str}})

    def test_list_of_document_field_jsonify(self):
        dt_now = datetime.now()
        dt_now_str = dt_now.strftime('%Y-%m-%dT%H:%M:%S')

        class ITD(BaseDoc):
            field1 = DateTimeField(default=dt_now)

        class TD(BaseDoc):
            field1 = ListField(DocumentField(ITD))

        td = TD(field1=[ITD(), ITD()])
        result = td.jsonify()
        self.assertEqual(
            result['field1'], [{'field1': dt_now_str}, {'field1': dt_now_str}])

    def test_from_mongo(self):
        class TD(BaseDoc):
            field1 = StringField()
        td = TD.from_mongo({'field1': 'test_string'})
        assert td.field1 == 'test_string'

    def test_from_mongo_hierarchy(self):
        dt_now = datetime.now()

        class ITD(BaseDoc):
            field1 = DateTimeField(default=dt_now)

        class TD(BaseDoc):
            field1 = StringField()
            field2 = DocumentField(ITD)

        td = TD.from_mongo({
            'field1': 'test_string',
            'field2': {'field1': dt_now}})
        assert isinstance(td.field2, ITD)
        assert td.field2.field1 == dt_now

    def test_from_mongo_hierarchy_list(self):
        dt_now = datetime.now()

        class ITD(BaseDoc):
            field1 = DateTimeField(default=dt_now)

        class TD(BaseDoc):
            field1 = StringField()
            field2 = ListField(DocumentField(ITD))

        td = TD.from_mongo({
            'field1': 'test_string',
            'field2': [{'field1': dt_now}]})
        assert isinstance(td.field2, list)
        assert td.field2[0].field1 == dt_now

    def test_from_mongo_hierarchy_dict(self):
        dt_now = datetime.now()

        class ITD(BaseDoc):
            field1 = DateTimeField(default=dt_now)

        class TD(BaseDoc):
            field1 = StringField()
            field2 = DictField(DocumentField(ITD))

        td = TD.from_mongo({
            'field1': 'test_string',
            'field2': {'a': {'field1': dt_now}}})
        assert isinstance(td.field2, dict)
        assert td.field2['a'].field1 == dt_now

    def test_from_mongo_deep_hierarchy(self):
        dt_now = datetime.now()

        class InListTD(BaseDoc):
            field1 = DateTimeField(default=dt_now)

        class InDictTD(BaseDoc):
            field1 = DateTimeField(default=dt_now)
            field2 = ListField(DocumentField(InListTD))

        class TD(BaseDoc):
            field1 = StringField()
            field2 = DictField(DocumentField(InDictTD))

        td = TD.from_mongo({
            'field1': 'test_string',
            'field2': {
                'a': {
                    'field1': dt_now,
                    'field2': [{'field1': dt_now}]}}})
        assert isinstance(td.field2, dict)
        assert isinstance(td.field2['a'].field2, list)
        assert isinstance(td.field2['a'].field2[0], InListTD)
        assert td.field2['a'].field2[0].field1 == dt_now

    def test_copy_in_place(self):
        class ITD(BaseDoc):
            field11 = IntField()

        class TD(BaseDoc):
            field1 = StringField()
            field2 = IntField()
            field3 = DocumentField(ITD)

        itd1 = ITD(field11=11)
        itd2 = ITD(field11=12)
        td1 = TD(field1='a', field2=1, field3=itd1)
        td2 = TD(field1='b', field2=2, field3=itd2)
        td1.copy_in_place(td2)

        # check source instance is unchanged
        assert td2.field1 == 'b'
        assert td2.field2 == 2
        assert td2.field3 is itd2

        # check target instance is a copy of source
        assert td1.field1 == td2.field1
        assert td1.field2 == td2.field2
        assert td1.field3.field11 == td2.field3.field11

    def test_required_fields_should_not_collide(self):
        class TD1(BaseDoc):
            field1 = StringField()
        td1 = TD1()
        td1.validate()
        assert True

    def test_required_fields_should_not_collide_in_one_cls(self):
        class TD(BaseDoc):
            field1 = StringField(required=True)
            field2 = StringField()
            field3 = StringField(required=True)

        td1 = TD(field1='test_string', field3='test_string')
        td1.validate()
        assert True

    def test_should_be_registered_in_registry(self):
        class Foo(BaseDoc):
            bar = StringField()
        class_fqn = '{}.{}'.format(
            inspect.getmodule(Foo).__name__, Foo.__name__)
        assert class_fqn in documents_registry
        assert documents_registry[class_fqn] == Foo

    def test_does_not_exist_exception(self):
        class TD1(BaseDoc):
            objects = Collection(db_name='main', collection_name='test_models1')

        class TD2(BaseDoc):
            objects = Collection(db_name='main', collection_name='test_models2')

        self.assertRaises(TD1.DoesNotExist, TD1.objects.get_or_raise, ObjectId())
        self.assertRaises(ObjectDoesNotExist, TD1.objects.get_or_raise, ObjectId())
        try:
            TD1.objects.get_or_raise(ObjectId())
        except TD2.DoesNotExist:
            self.fail("Collection raised DoesNotExist wrong exception")
        except TD1.DoesNotExist:
            # Right exception is raised
            assert True


class TestCollection(unittest.TestCase):
    def setUp(self):
        class _TestModel(BaseDoc):
            field1 = StringField(required=True)
            field2 = DateTimeField(default=datetime.now)

        self._TestModel = _TestModel
        self._TestModel.objects.indexes = [
            {
                'index': [('field2', 1), ],
                'background': True,
            },
            {
                'index': [('field1', 1), ('field2', -1), ],
                'background': True,
                'unique': True,
            }
        ]
        self._TestModel.objects.remove()
        self._get_collection().drop_indexes()

    def _get_collection(self):
        return self._TestModel.objects.collection

    def create_doc(self, payload):
        return self._TestModel.objects.model_class(**payload)

    def test_save_validate_ok(self):
        md = self._TestModel(field1='asasddsd')
        self._TestModel.objects.save(md)
        assert hasattr(md, '_id')

    def test_right_collection_should_be_used(self):
        col_name = self._TestModel.objects.collection_name
        assert col_name == 'test_models'

    def test_save_validate_failed(self):
        md = self._TestModel()
        self.assertRaises(ValidationError, self._TestModel.objects.save, md)

    def test_create(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        count = self._TestModel.objects.count()
        assert count == 1
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        count = self._TestModel.objects.count()
        assert count == 2

    def test_save_existing_doc(self):
        document = self._TestModel.objects.create({
            'field1': 'test_string'
        })
        document.field1 = 'test_string2'
        count = self._TestModel.objects.count()
        assert count == 1
        self._TestModel.objects.save(document)
        count = self._TestModel.objects.count()
        assert count == 1
        assert document.field1 == 'test_string2'
        stored_doc = self._TestModel.objects.get(document.id)
        assert stored_doc.field1 == 'test_string2'

    def test_insert(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        count = self._TestModel.objects.count()
        assert count == 1

        self._TestModel.objects.insert(self.create_doc({
            'field1': 'test_string'}))
        assert self._TestModel.objects.count() == 2

        self._TestModel.objects.insert([
            self.create_doc({'field1': 'test_string2'}),
            self.create_doc({'field1': 'test_string3'}),
        ])
        assert self._TestModel.objects.count() == 4

    def test_find(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        result = self._TestModel.objects.find().as_list()
        assert len(result) == 3

    def test_find_with_projection(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        result = self._TestModel.objects.find(fields={'field2': 1})
        assert hasattr(result[0], 'field2')
        self.assertIsNone(result[0].field1)

    def test_find_slice(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        result = self._TestModel.objects.find(skip=1, limit=2).as_list()
        assert len(result) == 2

    def test_slice(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        result = self._TestModel.objects.find()
        assert len(result[1:3]) == 2

    def test_find_and_modify_found(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        result = self._TestModel.objects.find_and_modify({
            'field1': 'test_string'}, {'$set': {'field1': 'updated'}})
        assert result.field1 == 'test_string'
        object_in_db = self._TestModel.objects.get({'field1': 'updated'})
        assert object_in_db.field1 == 'updated'

    def test_find_and_modify_not_found(self):
        result = self._TestModel.objects.find_and_modify({
            'field1': 'test_string'}, {'$set': {'field1': 'updated'}})
        assert result is None

    def test_find_and_modify_new_true(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        result = self._TestModel.objects.find_and_modify({
            'field1': 'test_string'}, {'$set': {'field1': 'updated'}},
            new=True)
        assert result.field1 == 'updated'
        object_in_db = self._TestModel.objects.get({'field1': 'updated'})
        assert object_in_db.field1 == 'updated'

    def test_remove_by_doc(self):
        document = self._TestModel.objects.create({
            'field1': 'test_string'
        })
        count = self._TestModel.objects.count()
        assert count == 1
        self._TestModel.objects.remove(document.id)
        count = self._TestModel.objects.count()
        assert count == 0

    def test_remove_by_id(self):
        document = self._TestModel.objects.create({
            'field1': 'test_string'
        })
        count = self._TestModel.objects.count()
        assert count == 1
        self._TestModel.objects.remove(document)
        count = self._TestModel.objects.count()
        assert count == 0

    def test_remove_by_spec(self):
        self._TestModel.objects.create({
            'field1': 'test_string'
        })
        self._TestModel.objects.create({
            'field1': 'test_string2'
        })
        count = self._TestModel.objects.count()
        assert count == 2
        self._TestModel.objects.remove({'field1': 'test_string'})
        count = self._TestModel.objects.count()
        assert count == 1

    def test_synchronization_index(self):
        def get_index_infos():
            indexes = self._get_collection().index_information()
            indexes.pop('_id_')
            return tuple(
                (tuple(indexes[i]['key']),
                 indexes[i].get('background', False),
                 indexes[i].get('expireAfterSeconds'))
                for i in indexes.keys()
            )

        def get_indexes():
            indexes = self._TestModel.objects.indexes
            return tuple(
                (tuple(i['index']),
                 i['background'],
                 i.get('expire_after_seconds'))
                for i in indexes
            )

        def get_indexes_to_change():
            indexes = self._TestModel.objects.synchronize_indexes()
            return (tuple(indexes['indexes_to_drop']),
                    tuple(indexes['indexes_to_ensure']))

        change = get_indexes_to_change()
        expected = ((), ('field2_1', 'field1_1_field2_-1'))
        to_set = lambda tpls: tuple(set(tpl) for tpl in tpls)

        self.assertEqual(
            to_set(change),
            to_set(expected)
        )
        assert set(get_index_infos()) == set(get_indexes())

        self._get_collection().drop_index(
            'field1_1_field2_-1')
        change = get_indexes_to_change()
        assert change, ((), ('field1_1_field2_-1', ))

        assert set(get_index_infos()) == set(get_indexes())

        self._TestModel.objects.indexes[1]['index'][0] = ('field13', -1)
        self._TestModel.objects.indexes[1]['background'] = False
        change = get_indexes_to_change()
        assert change == (('field1_1_field2_-1',), ('field13_-1_field2_-1',))
        assert set(get_index_infos()) == set(get_indexes())

    def test_ttl(self):
        self._TestModel.objects.synchronize_indexes()

        self._TestModel.field1 = DateTimeField
        self._TestModel.objects.indexes[1]['expire_after_seconds'] = 1
        with pytest.raises(TypeError):
            self._TestModel.objects.synchronize_indexes()

        self._TestModel.objects.indexes[1].pop('expire_after_seconds')
        self._TestModel.objects.synchronize_indexes()

        self._TestModel.objects.indexes[0]['expire_after_seconds'] = '1'
        with pytest.raises(TypeError):
            self._TestModel.objects.synchronize_indexes()

        self._TestModel.objects.indexes[0]['expire_after_seconds'] = 1
        self._TestModel.field2 = StringField

        try:
            self._TestModel.objects.synchronize_indexes()
            raise Exception
        except Exception as ex:
            assert type(ex) == TypeError

    def test_unique(self):
        self._TestModel.objects.synchronize_indexes()

        dt = datetime.utcnow()
        dt2 = dt + timedelta(days=1)

        self._TestModel.objects.create(dict(
            field1='1', field2=dt
        ))
        self._TestModel.objects.create(dict(
            field1='2', field2=dt
        ))
        self._TestModel.objects.create(dict(
            field1='1', field2=dt2
        ))
        self._TestModel.objects.create(dict(
            field1='2', field2=dt2
        ))

        with pytest.raises(DuplicateKeyError):
            self._TestModel.objects.create(dict(
                field1='1', field2=dt
            ))

        self._TestModel.objects.indexes[1]['unique'] = False
        self._TestModel.objects.synchronize_indexes()

        self._TestModel.objects.create(dict(
            field1='3', field2=dt
        ))

        self._TestModel.objects.create(dict(
            field1='3', field2=dt
        ))

    def test_create_document(self):

        class ITD(BaseDoc):
            int_field = IntField()
            string_field = StringField()

        class TD(BaseDoc):
            dict_field = DictField(DocumentField(ITD))
            list_field = ListField(DocumentField(ITD))
            model_field = DocumentField(ITD)

        td = TD.objects.create({
            'dict_field': {'1': {'int_field': 1, 'string_field': '2'}},
            'list_field': [{'int_field': 3, 'string_field': '4'}],
            'model_field': {'int_field': 5, 'string_field': '6'}
        })

        assert isinstance(td.dict_field['1'], ITD)
        assert isinstance(td.list_field[0], ITD)
        assert isinstance(td.model_field, ITD)


class TestInheritance(unittest.TestCase):
    def setUp(self):
        class TD1(BaseDoc):
            required_field1 = StringField(required=True)
            default_field1 = StringField(default='df1')
            choices_field1 = StringField(choices=ContactStatus)
            simple_field1 = StringField()

        class TD2(BaseDoc):
            required_field2 = StringField(required=True)
            default_field2 = StringField(default='df2')
            choices_field2 = StringField(choices=ContactStatus)
            simple_field2 = StringField()

        class TD3(TD1, TD2):
            required_field3 = StringField(required=True)
            default_field3 = StringField(default='df3')
            choices_field3 = StringField(choices=ContactStatus)
            simple_field3 = StringField()

        self.TD3 = TD3
        self.TD3Collection = TD3.objects
        self.TD3Collection.remove()
        self.TD3Collection.collection.drop_indexes()

    def test_correct_save_required_fields(self):
        self.TD3Collection.create({
            'required_field1': 'test_string1',
            'required_field2': 'test_string2',
            'required_field3': 'test_string3'
        })
        td = self.TD3Collection.get()
        assert td.required_field1 == 'test_string1'
        assert td.required_field2 == 'test_string2'
        assert td.required_field3 == 'test_string3'

    def test_incorrect_save_required_fields(self):
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({})
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({'required_field1': 'test_string1'})
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({
                'required_field1': 'test_string1',
                'required_field2': 'test_string2'})
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({
                'required_field2': 'test_string2',
                'required_field3': 'test_string3'})
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({
                'required_field1': 'test_string1',
                'required_field3': 'test_string3'})

    def test_correct_save_default_value_to_default_fields(self):
        self.TD3Collection.create({
            'required_field1': 'test_string1',
            'required_field2': 'test_string2',
            'required_field3': 'test_string3'
        })
        td = self.TD3Collection.get()
        assert td.default_field1 == 'df1'
        assert td.default_field2 == 'df2'
        assert td.default_field3 == 'df3'

    def test_correct_save_random_value_to_default_fields(self):
        self.TD3Collection.create({
            'required_field1': 'test_string1',
            'required_field2': 'test_string2',
            'required_field3': 'test_string3',
            'default_field1': 'ty1',
            'default_field2': 'ty2',
            'default_field3': 'ty3',
        })
        td = self.TD3Collection.get()
        assert td.default_field1 == 'ty1'
        assert td.default_field2 == 'ty2'
        assert td.default_field3 == 'ty3'

    def test_correct_save_choices_fields(self):
        self.TD3Collection.create({
            'required_field1': 'test_string1',
            'required_field2': 'test_string2',
            'required_field3': 'test_string3',
            'choices_field1': ContactStatus.Done,
            'choices_field2': ContactStatus.Failed,
            'choices_field3': ContactStatus.Awaiting,
        })
        td = self.TD3Collection.get()
        assert td.choices_field1 == ContactStatus.Done
        assert td.choices_field2 == ContactStatus.Failed
        assert td.choices_field3 == ContactStatus.Awaiting

    def test_incorrect_save_choices_fields(self):
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({
                'required_field1': 'test_string1',
                'required_field2': 'test_string2',
                'required_field3': 'test_string3',
                'choices_field1': 'r3',
                'choices_field2': ContactStatus.Failed,
                'choices_field3': ContactStatus.Awaiting,
            })
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({
                'required_field1': 'test_string1',
                'required_field2': 'test_string2',
                'required_field3': 'test_string3',
                'choices_field1': ContactStatus.Failed,
                'choices_field2': '43',
                'choices_field3': ContactStatus.Awaiting,
            })
        with self.assertRaises(ValidationError):
            self.TD3Collection.create({
                'required_field1': 'test_string1',
                'required_field2': 'test_string2',
                'required_field3': 'test_string3',
                'choices_field1': ContactStatus.Failed,
                'choices_field2': ContactStatus.Failed,
                'choices_field3': '43',
            })

    def test_correct_save_simple_fields(self):
        self.TD3Collection.create({
            'required_field1': 'test_string1',
            'required_field2': 'test_string2',
            'required_field3': 'test_string3',
            'simple_field1': 'ty1',
            'simple_field2': 'ty2',
            'simple_field3': 'ty3',
        })
        td = self.TD3Collection.get()
        assert td.simple_field1 == 'ty1'
        assert td.simple_field2 == 'ty2'
        assert td.simple_field3 == 'ty3'


class TestEmailValidator(unittest.TestCase):
    def setUp(self):
        self.validator = EmailValidator()

    def test_valid_emails(self):
        valid_emails = [
            'some@email.r',
            '"Abc\@def"@example.com',
            '"Fred Bloggs"@example.com',
            '"Joe\\Blow"@example.com',
            '"Abc@def"@example.com',
            'customer/department=shipping@example.com',
            '\$A12345@example.com',
            '!def!xyz%abc@example.com',
            '_somename@example.com',
            'test+label@example.com',
            'mimimi@президент.рф'
        ]
        for email in valid_emails:
            assert self.validator.validate(email)

    def test_invalid_emails(self):
        invalid_emails = ['email.r', '&&&abir@valg']
        for email in invalid_emails:
            assert not self.validator.validate(email)



import unittest
from datetime import datetime

from ognom.base import Collection
from ognom.fields import StringField
from ognom.helpers.lifecycle import DocWithLifeCycle


class FooDoc(DocWithLifeCycle):
    objects = Collection(
        db_name='main',
        collection_name='testmodel'
    )

    bar = StringField()


class TestLifeCycleMixin(unittest.TestCase):

    def test_created_at_should_be_inplace(self):
        td = FooDoc.objects.create({'bar': 'baz'})
        assert isinstance(td.created_at, datetime)
        assert td.bar == 'baz'

    def test_updated_at_should_not_be_inplace(self):
        td = FooDoc.objects.create({})
        assert td.updated_at is None

    def test_updated_at_should_present(self):
        td = FooDoc.objects.create({})
        td.updated_at = datetime.utcnow()
        td.save()
        assert isinstance(td.updated_at, datetime)

    def test_deleted_at_should_present(self):
        td = FooDoc.objects.create({})
        td.deleted_at = datetime.utcnow()
        td.save()
        assert isinstance(td.deleted_at, datetime)



from __future__ import unicode_literals
import logging
from time import sleep

from six import string_types
from pymongo import MongoReplicaSetClient, MongoClient
from pymongo.errors import ConnectionFailure


MAX_ATTEMPTS = 20
CONNECT_SLEEP_TIME = 1


class ConnectionManager(object):
    databases = {}
    connections = {}

    @classmethod
    def connect(cls, mongo_settings):
        for db_alias, settings in mongo_settings.items():
            db_name = settings['name']
            connection = cls._get_connection(settings)
            cls.connections[db_alias] = connection
            cls.databases[db_alias] = getattr(connection, db_name)

    @classmethod
    def get_database(cls, db_alias):
        return cls.databases.get(db_alias)

    @classmethod
    def get_databases(cls):
        return cls.databases.items()

    @classmethod
    def get_connections(cls):
        return cls.connections.items()

    @classmethod
    def drop_database(cls, db_alias):
        connection = cls.databases[db_alias]
        cls.connections[db_alias].drop_database(connection.name)

    @classmethod
    def _get_connection(cls, configuration):
        if isinstance(configuration, string_types):
            connection = cls._establish_connection(configuration)
        elif isinstance(configuration, list):
            connection = cls._establish_connection(*configuration)
        elif isinstance(configuration, dict):
            connection = cls._establish_connection(
                *configuration.get('args', []),
                **configuration.get('kwargs', {}))
        else:
            raise ValueError(
                'Unsupported configuration format: %s' % configuration)
        return connection

    @classmethod
    def _establish_connection(cls, *args, **kwargs):
        for i in range(MAX_ATTEMPTS - 1):
            try:
                if 'replicaSet' in kwargs:
                    return MongoReplicaSetClient(*args, **kwargs)
                else:
                    return MongoClient(*args, **kwargs)
            except ConnectionFailure as e:
                logging.error(
                    'Fail to connect to %s, %s [%s]', args, kwargs, e)
                sleep(CONNECT_SLEEP_TIME)
        if 'replicaSet' in kwargs:
            return MongoReplicaSetClient(*args, **kwargs)
        else:
            return MongoClient(*args, **kwargs)



from datetime import datetime

from ognom.base import Document
from ognom.fields import DateTimeField


class DocWithLifeCycle(Document):
    created_at = DateTimeField(default=datetime.utcnow)
    updated_at = DateTimeField()
    deleted_at = DateTimeField()



import inspect

documents_registry = {}


class NoDocumentClassError(Exception):
    pass


def register_doc_class(doc_cls):
    global documents_registry
    module_fqn = inspect.getmodule(doc_cls).__name__
    documents_registry['{}.{}'.format(module_fqn, doc_cls.__name__)] = doc_cls


def get_doc_class(doc_name):
    if doc_name not in documents_registry:
        raise NoDocumentClassError(doc_name)
    return documents_registry[doc_name]



from __future__ import unicode_literals
import copy

from bson import ObjectId
from six import with_metaclass

from ognom.fields import GenericField, ObjectIdField, ValidationError
from ognom._registry import register_doc_class


class ObjectDoesNotExist(Exception):
    pass


class MongoDocumentMeta(type):
    def __new__(cls, class_name, bases, dct):
        result_cls = type.__new__(cls, class_name, bases, dct)
        register_doc_class(result_cls)
        if result_cls.objects is not None:
            result_cls.objects.model_class = result_cls

        result_cls._defaults = {}
        result_cls._choices = {}
        result_cls._required = set()
        result_cls.DoesNotExist = type(
            str('DoesNotExist'), (ObjectDoesNotExist,), {})

        # in order to support inheritance
        special_attrs = [
            ('_defaults', {}), ('_choices', {}), ('_required', set())]
        for field, default in special_attrs:
            # copy values of specific fields from parent
            for base in bases:
                base_field_value = getattr(base, field, None)
                if base_field_value is not None:
                    current_field_value = getattr(result_cls, field)
                    if isinstance(base_field_value, dict):
                        current_field_value.update(base_field_value)
                    elif isinstance(base_field_value, set):
                        current_field_value |= base_field_value
                    else:
                        raise NotImplementedError(field)

        for name, value in result_cls.__dict__.items():
            if isinstance(value, GenericField):
                if not value.name:
                    value.name = name
                if value.choices:
                    result_cls._choices[name] = set(value.choices)
                if value.required:
                    result_cls._required.add(name)
                if value.default is not None:
                    result_cls._defaults[name] = value.default
        return result_cls


class Document(with_metaclass(MongoDocumentMeta, object)):
    _id = ObjectIdField()

    _required = None
    _defaults = None
    _choices = None
    DoesNotExist = None

    objects = None  # repository can be injected here

    def __new__(cls, *args, **kwargs):
        instance = super(Document, cls).__new__(cls)
        instance._data = {}
        instance.apply_defaults()
        return instance

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)

    @property
    def id(self):
        return self._id

    def apply_defaults(self):
        for key, value in self._defaults.items():
            if key not in self._data:
                if callable(value):
                    self._data[key] = value()
                else:
                    self._data[key] = value

    def validate(self):
        for required_field in self._required:
            if self._data.get(required_field) is None:
                raise ValidationError(
                    'Field {} is missing'.format(required_field),
                    required_field)

        for name, choices in self._choices.items():
            if (name in self._data and
                    self._data[name] is not None and
                    self._data[name] not in choices):
                raise ValidationError(
                    'Field {} value {} is not included in {}'.format(
                        name, self._data[name], choices))

        for name, value in self._data.items():
            if value is None and name not in self._required:
                continue

            attribute = self.__class__.__dict__.get(name)
            if attribute:
                attribute.validate(value)

    def to_mongo(self):
        result = {}
        for name, value in self._data.items():
            if value is None:
                continue

            attribute = getattr(self.__class__, name, None)
            if attribute:
                result[name] = attribute.to_mongo(value)
            elif name == '_id' and value:
                result[name] = value
        return result

    def _get_prepared_data(self):
        return self._data

    def jsonify(self):
        result = {}
        for name, value in self._get_prepared_data().items():
            if value is None:
                continue
            attribute = self.__class__.__dict__.get(name)
            if attribute and hasattr(attribute, 'jsonify'):
                result[name] = attribute.jsonify(value)
            elif name == '_id' and value:
                result['id'] = str(value)
            else:
                result[name] = value
        return result

    @classmethod
    def from_mongo(cls, payload):
        instance = None
        if payload:
            instance = cls()
            for key, value in payload.items():
                attribute = getattr(instance.__class__, key, None)
                if attribute and hasattr(attribute, 'from_mongo'):
                    setattr(instance, key, attribute.from_mongo(value))
                elif key == '_id' and value:
                    setattr(instance, key, value)
        return instance

    @classmethod
    def from_json(cls, payload):
        payload_ = {}
        if payload:
            for key, value in payload.items():
                attribute = cls.__dict__.get(key)
                if attribute and isinstance(attribute, GenericField):
                    try:
                        payload_[key] = attribute.from_json(value)
                    except (ValueError, TypeError) as ex:
                        raise ValidationError(repr(ex), key)
                elif key in ('id', '_id') and value:
                    payload_['_id'] = ObjectId(value)
        instance = cls(**payload_)
        return instance

    def copy_in_place(self, instance):
        self._data = copy.deepcopy(instance._data)

    def save(self):
        if self.objects:
            return self.objects.save(self)
        raise NotImplemented('objects attribute was not specified')

    def remove(self):
        if self.objects:
            return self.objects.remove(self)
        raise NotImplemented('objects attribute was not specified')

    def copy(self):
        data = copy.deepcopy(self._data)
        data.pop('_id')
        return self.__class__(**data)

    def __repr__(self):
        return '{self.__class__.__name__}:{self.id}'.format(self=self)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False

        if self.id is None:  # can't compare models without id's
            return False
        return self.id == other.id

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        if self.id is None:
            raise TypeError('Documents without id are unhashable')
        return hash(self.id)



from __future__ import unicode_literals
from unittest import TestCase

from pymongo import MongoClient

from ognom.helpers.counter import MongoCounter


class TestMongoCounter(TestCase):
    def setUp(self):
        self.mongo_client = MongoClient(host='localhost:27017')
        self.collection = self.mongo_client.test_mongo_counters.counters
        self.collection.remove()

    def test_valid(self):
        counter = MongoCounter(self.collection, 'counter1')
        assert counter.next_value() == 1
        assert counter.next_value() == 2
        assert counter.get_value() == 2
        assert counter.next_value() == 3
        assert counter.get_value() == 3



from __future__ import unicode_literals
import re
import sys
import uuid
import datetime
import decimal

import six
from bson import ObjectId
from dateutil import parser

from ognom._registry import get_doc_class


class ValidationError(Exception):
    def __init__(self, message, field_name=None, *args, **kwargs):
        super(ValidationError, self).__init__(message)
        self.message = message
        self.field_name = field_name


class GenericField(object):
    def __init__(self, required=False, default=None, choices=None,
                 validators=None):
        self.required = required
        self.default = default
        self.name = None
        self.choices = choices
        self.validators = validators if validators else []

    def __get__(self, instance, owner):
        if not instance:
            return self
        return instance._data.get(self.name)

    def __set__(self, instance, value):
        instance._data[self.name] = self.prepare_to_assign(value)

    def prepare_to_assign(self, value):
        return value

    def validate(self, value):
        if value:
            for validator in self.validators:
                if not validator.validate(value):
                    raise ValidationError(validator.MESSAGE.format(
                        field=self.name, value=value), self.name)

    def to_mongo(self, value):
        return value

    def jsonify(self, value):
        return value

    def from_mongo(self, value):
        return value

    def from_json(self, value):
        """
        Most of the fields use BSON compatible types,
        so we only need to cast string to UUID/ObjectId/DateTime.
        We need smart casting only in case of Complex user defined types.
        """
        return self.to_mongo(value)


class StringField(GenericField):
    def validate(self, value):
        super(StringField, self).validate(value)
        if value and not isinstance(value, six.string_types):
            raise ValidationError(
                '[{}]: String accepts only string values, '
                '{} of {} given!'.format(
                    self.name, value, type(value)), self.name)

    def jsonify(self, value):
        return value


class URLField(StringField):
    _URL_REGEX = re.compile(
        r'^(?:http|ftp)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|'
        r'[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    def validate(self, value):
        super(URLField, self).validate(value)
        if value and not self._URL_REGEX.match(value):
            raise ValidationError(
                '[{}] Invalid url {}'.format(self.name, value), self.name)


class HTTPField(URLField):
    _URL_REGEX = re.compile(
        r'^(?:http)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|'
        r'[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)


class IntField(GenericField):
    """An 32-bit integer field."""
    def validate(self, value):
        super(IntField, self).validate(value)
        if value is not None and not isinstance(value, six.integer_types):
            raise ValidationError(
                '[{}] Couldn\'t convert {} to int'.format(
                    self.name, value), self.name)

    def to_mongo(self, value):
        if value is not None:
            return int(value)
        return value


class FloatField(GenericField):
    def validate(self, value):
        super(FloatField, self).validate(value)
        if not isinstance(value, (float, decimal.Decimal, six.integer_types)):
            raise ValidationError(
                '[{}] Can\'t convert {} of type {} to float'.format(
                    self.name, value, type(value)), self.name)

    def to_mongo(self, value):
        return float(value)


class DecimalField(GenericField):
    def validate(self, value):
        super(DecimalField, self).validate(value)
        try:
            decimal.Decimal(value)
        except decimal.InvalidOperation:
            raise ValidationError(
                '[{}] Can\'t convert {} of type {} to Decimal'.format(
                    self.name, value, type(value)), self.name)

    def to_mongo(self, value):
        return six.text_type(value)

    def from_mongo(self, value):
        return decimal.Decimal(value)


class UUIDField(GenericField):
    def to_mongo(self, value):
        if isinstance(value, six.string_types):
            return uuid.UUID(value)
        return value

    def validate(self, value):
        super(UUIDField, self).validate(value)
        if not isinstance(value, uuid.UUID):
            raise ValidationError(
                '[{}] Cannot convert to UUID {}'.format(
                    self.name, value), self.name)

    def jsonify(self, value):
        return str(value)


class ObjectIdField(GenericField):
    def validate(self, value):
        super(ObjectIdField, self).validate(value)
        if not isinstance(value, ObjectId):
            raise ValidationError(
                '[{}] Invalid value for ObjectId {}'.format(
                    self.name, value), self.name)

    def to_mongo(self, value):
        if not isinstance(value, ObjectId):
            try:
                return ObjectId(six.text_type(value))
            except Exception as ex:
                raise ValidationError(
                    '[{}] Invalid value for ObjectId {}. Error: {}'.format(
                        self.name, value, repr(ex)), self.name)
        return value

    def jsonify(self, value):
        return str(value)


class DateTimeField(GenericField):
    def validate(self, value):
        super(DateTimeField, self).validate(value)
        if value and not isinstance(value, datetime.datetime):
            try:
                parser.parse(value)
            except Exception as ex:
                raise ValidationError(
                    '[{}] Unable to convert {} to datetime: {}'.format(
                        self.name, value, repr(ex)))
        return True

    def to_mongo(self, value):
        if value is None:
            return value
        if isinstance(value, datetime.datetime):
            return value
        if isinstance(value, datetime.date):
            return datetime.datetime(value.year, value.month, value.day)

        if not isinstance(value, six.string_types):
            return None

        try:
            return parser.parse(value)
        except ValueError as ex:
            raise ValidationError(
                '[{}] Unable to convert {} to datetime: {}'.format(
                    self.name, value, repr(ex)))

    def jsonify(self, value):
        return value.strftime('%Y-%m-%dT%H:%M:%S%z')


class BooleanField(GenericField):
    def validate(self, value):
        if not isinstance(value, bool):
            raise ValidationError(
                '[{}] Only bool value can be used'.format(self.name),
                self.name)


class ListField(GenericField):
    def __init__(self, field_type, required=False, default=None,
                 validators=None):
        super(ListField, self).__init__(
            required, default, validators=validators)
        self.field_type = field_type

    def to_mongo(self, value):
        return [self.field_type.to_mongo(item) for item in value]

    def validate(self, value):
        super(ListField, self).validate(value)
        try:
            iter(value)
        except TypeError:
            raise ValidationError(
                u"[{}] Iterable expected {} with value {} was found.".format(
                    self.name, type(value), value),
                self.name)
        for item in value:
            self.field_type.validate(item)

    def jsonify(self, value):
        return [self.field_type.jsonify(item) for item in value]

    def from_mongo(self, value):
        return [self.field_type.from_mongo(v) for v in value]

    def from_json(self, value):
        return [self.field_type.from_json(v) for v in value]

    def prepare_to_assign(self, value):
        if value is not None:
            value = [
                self.field_type.prepare_to_assign(v) for v in value
            ]
        return value


class DictField(GenericField):
    def __init__(self, field_type, required=False, default=None,
                 validators=None):
        super(DictField, self).__init__(
            required, default, validators=validators)
        self.field_type = field_type

    def to_mongo(self, value):
        return {key: self.field_type.to_mongo(item)
                for key, item in value.items()}

    def validate(self, value):
        super(DictField, self).validate(value)
        if value is None:
            value = {}
        if not isinstance(value, dict):
            raise ValidationError(
                u"[{}] Dict expected {} with value {} was found.".format(
                    self.name, type(value), value),
                self.name)
        for item in value.values():
            self.field_type.validate(item)

    def jsonify(self, value):
        return {key: self.field_type.jsonify(item)
                for key, item in value.items()}

    def from_mongo(self, value):
        return {
            k: self.field_type.from_mongo(v)
            for k, v in value.items()
        }

    def from_json(self, value):
        return {
            k: self.field_type.from_json(v)
            for k, v in value.items()
        }

    def prepare_to_assign(self, value):
        if value is not None:
            for key, item in value.items():
                value[key] = self.field_type.prepare_to_assign(item)
        return value


class DocumentField(GenericField):
    def __init__(self, model_class, required=False, default=None,
                 validators=None):
        super(DocumentField, self).__init__(
            required, default, validators=validators)
        if isinstance(model_class, six.string_types):
            self._model_class_name = model_class
            self._model_class = None
        else:
            self._model_class = model_class

    def validate(self, value):
        if not isinstance(value, self.model_class):
            try:
                value = self.model_class(**value)
            except Exception:
                err_msg = u"[{}] {} or dict are accepted, {} given".format(
                    self.name, self.model_class, value
                )
                six.reraise(ValidationError, err_msg, sys.exc_info()[2])
        value.validate()

    def to_mongo(self, value):
        if not isinstance(value, self.model_class):
            value = self.model_class(**value)
        return value.to_mongo()

    def jsonify(self, value):
        if not hasattr(value, 'jsonify'):
            if isinstance(value, dict):
                value = self.model_class(**value)
            else:
                raise ValidationError(
                    u'[{}] Wrong object type passed: {}'.format(
                        self.name, type(value)))
        return value.jsonify()

    def from_mongo(self, payload):
        return self.model_class(**payload) if payload else None

    def from_json(self, value):
        return self.model_class.from_json(value)

    def prepare_to_assign(self, value):
        if value is not None and not isinstance(value, self.model_class):
            value = self.model_class(**value)
        return value

    @property
    def model_class(self):
        if self._model_class is None:
            self._model_class = get_doc_class(self._model_class_name)
        return self._model_class


class GenericDocumentField(GenericField):
    def __init__(self, attribute, attr_to_class, required=False, default=None):
        super(GenericDocumentField, self).__init__(required, default)
        self.attribute = attribute
        self.attr_to_class = attr_to_class

    def validate(self, value):
        possible_types = tuple(self.attr_to_class.values())
        if not isinstance(value, possible_types):
            value_class = self._get_class(value)
            try:
                value_obj = value_class(**value)
            except Exception:
                err_msg = u"[{}] {} or dict are accepted, {} given".format(
                    self.name, value_class, value
                )
                six.reraise(ValidationError, err_msg, sys.exc_info()[2])
        else:
            value_obj = value
        value_obj.validate()

    def to_mongo(self, value):
        possible_types = tuple(self.attr_to_class.values())
        if not isinstance(value, possible_types):
            value_class = self._get_class(value)
            value_obj = value_class(**value)
        else:
            value_obj = value
        return value_obj.to_mongo()

    def jsonify(self, value):
        if not hasattr(value, 'jsonify'):
            if isinstance(value, dict):
                value_class = self._get_class(value)
                value = value_class(**value)
            else:
                raise ValidationError(
                    u'[{}] Wrong object type was passed: {}'.format(
                        self.name, type(value)))
        return value.jsonify()

    def from_mongo(self, payload):
        if payload:
            value_class = self._get_class(payload)
            return value_class(**payload)
        return None

    def from_json(self, value):
        value_class = self._get_class(value)
        return value_class.from_json(value)

    def prepare_to_assign(self, value):
        possible_types = tuple(self.attr_to_class.values())
        if value is not None and not isinstance(value, possible_types):
            value_class = self._get_class(value)
            value = value_class(**value)
        return value

    def _get_class(self, value):
        attribute_value = value.get(self.attribute)
        value_class = self.attr_to_class.get(attribute_value)
        if not value_class:
            raise ValidationError(
                u'No class for {}:{}'.format(self.attribute, attribute_value))
        return value_class


class IdField(ObjectIdField):
    name = '_id'



from __future__ import unicode_literals
import re


class AbstractValidator(object):
    MESSAGE = 'Abstract message'

    def validate(self, value):
        raise NotImplementedError(
            'Validate method must be implemented for the class {}'.format(
                self.__class__))


class LengthValidator(AbstractValidator):
    MESSAGE = 'Length of value for field "{field}" must be lte then {value}'

    def __init__(self, length):
        self.length = length

    def validate(self, value):
        return len(value) <= self.length


class EmailValidator(AbstractValidator):
    MESSAGE = 'Value of the field "{field}" is not valid email: {value}'
    _EMAIL_REGEXP = re.compile(r'.+@.+\..+$', re.IGNORECASE)

    def validate(self, value):
        return self._EMAIL_REGEXP.match(value) is not None


class TimeValidator(AbstractValidator):
    MESSAGE = 'Value of the field "{field}" is not valid "hh:mm": {value}'
    _TIME_REGEXP = re.compile(r'^(\d{1,2}):(\d{1,2})$', re.IGNORECASE)

    def validate(self, value):
        result = self._TIME_REGEXP.match(value)
        if result is not None:
            hours, minutes = map(int, result.group(1, 2))
            if hours == 24 and minutes == 0:  # special case to define full day
                return True
            else:
                return 0 <= hours <= 23 and 0 <= minutes <= 59
        else:
            return False



#!/usr/bin/env python
import os
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

from ognom import __version__


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ""


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', 'Arguments to pass to py.test')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ['tests/']

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

setup(
    name='ognom',
    description='Neat ODM wrapper around great PyMongo driver',
    long_desccription=read('README.md'),

    author='Lev Orekhov',
    author_email='lev.orekhov@gmail.com',
    url='https://github.com/lorehov/ognom',

    maintainer='Lev Orekhov',
    maintainer_email='lev.orekhov@gmail.com',


    version=__version__,
    packages=find_packages(),

    tests_require=['pytest', 'pymongo', 'six'],
    install_requires=['pymongo', 'six'],
    cmdclass={'test': PyTest},

    keywords='mongo mongodb pymongo orm odm',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    license='MIT',
)
